#include "src/include.h"

/*--------------------------------------------------------*/
/* Calculation of the observables in the Standard Model   */
/*--------------------------------------------------------*/

int main(int argc,char** argv)
{ 
	struct relicparam paramrelic;
	double eta,H2_H,He3_H2,Yp,Li7_H,Li6_Li7,Be7_H;
	double Trh;
	double dd0,ndd;
	double Sigmad0,nSigmad;

	if(argc<6) 
  	{ 
    		printf(" This program needs 5 parameter:\n"
 		"   dd0     dark energy proportion at BBN time\n"
		"   ndd     dark energy decrease exponent (preferentially >4)\n"
		"   Sigmad0 dark entropy production proportion at BBN time\n"
		"   nSigmad dark entropy production decrease exponent (-1 for standard reheating)\n"
          	"   Trh     reheating temperature (in GeV)\n");    
      		exit(1); 
  	} 
	else 
  	{
   		sscanf(argv[1],"%lf",&dd0);
     		sscanf(argv[2],"%lf",&ndd);
     		sscanf(argv[3],"%lf",&Sigmad0);
     		sscanf(argv[4],"%lf",&nSigmad);
    		sscanf(argv[5],"%lf",&Trh);
  	}
	
	Init_cosmomodel(&paramrelic);	

	Init_dark_density(dd0,ndd,Trh,&paramrelic);
	Init_dark_entropySigmaD(Sigmad0,nSigmad,Trh,&paramrelic);
	
	nucl(&paramrelic,&eta,&H2_H,&He3_H2,&Yp,&Li7_H,&Li6_Li7,&Be7_H);
	
	printf("Yp\t\t H2/H\t\t He3/H2\t\t Li7/H\t\t Li6/Li7\t Be7/H\n");
	printf("%.3e\t %.3e\t %.3e\t %.3e\t %.3e\t %.3e\n",Yp,H2_H,He3_H2,Li7_H,Li6_Li7,Be7_H);

	int compat=bbn_excluded(&paramrelic);

	if(compat==1) printf("Excluded by BBN constraints\n");
	else if(compat==0) printf("Compatible with BBN constraints\n");
	else printf("Computation failed\n");

	return 1;
}
