#include "src/include.h"

/*--------------------------------------------------------*/
/* Calculation of the observables in the Standard Model   */
/*--------------------------------------------------------*/

int main(int argc,char** argv)
{ 
	struct relicparam paramrelic;
	double eta,H2_H,He3_H2,Yp,Li7_H,Li6_Li7,Be7_H;
	double dd0,ndd,Tdrh;
	double sd0,nsd,Tsrh;
	double nt0,nnt,Tnrh;

	if(argc<5) 
  	{ 
    		printf(" This program needs 4 parameter:\n"
		"   dd0     dark energy proportion at BBN time\n"
		"   ndd     dark energy decrease exponent (preferentially >4)\n"
		"   sd0     dark entropy proportion at BBN time\n"
		"   nsd     dark entropy decrease exponent\n");
		printf(" Auxiliary parameters are:\n"
           	"   Td      dark energy cut temperature (in GeV)\n"    
           	"   Ts      dark entropy cut temperature (in GeV)\n");
      		exit(1); 
  	} 
	else 
	{
  		sscanf(argv[1],"%lf",&dd0);
     		sscanf(argv[2],"%lf",&ndd);
     		sscanf(argv[3],"%lf",&sd0);
     		sscanf(argv[4],"%lf",&nsd);
		if(argc>5) sscanf(argv[5],"%lf",&Tdrh); else Tdrh=0.;
     		if(argc>6) sscanf(argv[6],"%lf",&Tsrh); else Tsrh=0.;   
  	}
	
	Init_cosmomodel(&paramrelic);	

	Init_dark_density(dd0,ndd,Tdrh,&paramrelic);
	Init_dark_entropy(sd0,nsd,Tsrh,&paramrelic);
	
	nucl(&paramrelic,&eta,&H2_H,&He3_H2,&Yp,&Li7_H,&Li6_Li7,&Be7_H);
	
	printf("Yp\t\t H2/H\t\t He3/H2\t\t Li7/H\t\t Li6/Li7\t Be7/H\n");
	printf("%.3e\t %.3e\t %.3e\t %.3e\t %.3e\t %.3e\n",Yp,H2_H,He3_H2,Li7_H,Li6_Li7,Be7_H);

	int compat=bbn_excluded(&paramrelic);

	if(compat==1) printf("Excluded by BBN constraints\n");
	else if(compat==0) printf("Compatible with BBN constraints\n");
	else printf("Computation failed\n");

	return 1;
}
