#include "include.h"

/*--------------------------------------------------------------*/

void Init_modeleff(int model_eff, struct relicparam* paramrelic)
{
	int ie,je;
	
	if(model_eff==1)
	{
		const double tableA[276][3]=
		{
#include "sgStar_heff/sgStar_heff_A.tab"
  		};
		for(ie=0;ie<=275;ie++) for(je=0;je<=2;je++) paramrelic->table_eff[ie][je]=tableA[ie][je];
	}
	else if(model_eff==2)
	{
		const double tableB[276][3]=
		{
#include "sgStar_heff/sgStar_heff_B.tab"
  		};
		for(ie=0;ie<=275;ie++) for(je=0;je<=2;je++) paramrelic->table_eff[ie][je]=tableB[ie][je];
	}
	else if(model_eff==3)
	{
	const double tableB2[276][3]=
		{
#include "sgStar_heff/sgStar_heff_B2.tab"
  		};
		for(ie=0;ie<=275;ie++) for(je=0;je<=2;je++) paramrelic->table_eff[ie][je]=tableB2[ie][je];
		}
	else if(model_eff==4)
	{
		const double tableB3[276][3]=
		{
#include "sgStar_heff/sgStar_heff_B3.tab"
  		};	
		for(ie=0;ie<=275;ie++) for(je=0;je<=2;je++) paramrelic->table_eff[ie][je]=tableB3[ie][je];
	}
	else if(model_eff==5)
	{
		const double tableC[276][3]=
		{
#include "sgStar_heff/sgStar_heff_C.tab"
  		};
		for(ie=0;ie<=275;ie++) for(je=0;je<=2;je++) paramrelic->table_eff[ie][je]=tableC[ie][je];
	}
	else
	{
		const double tableold[276][3]=
		{
#include "sgStar_heff/sgStar_heff_old.tab"
  		};
		for(ie=0;ie<=275;ie++) for(je=0;je<=2;je++) paramrelic->table_eff[ie][je]=tableold[ie][je];
	}
	return;
}

/*--------------------------------------------------------------*/

double heff(double Temp, struct relicparam* paramrelic)
{
	int ie;
	
	if(Temp>= paramrelic->table_eff[0][0]) return paramrelic->table_eff[0][2];
	
	if(Temp<=0.) return paramrelic->table_eff[275][2];

	ie=1;
	while(Temp<paramrelic->table_eff[ie][0]) ie++;
	
	double heff1,heff2,T1,T2;
	heff1=paramrelic->table_eff[ie][2];
	heff2=paramrelic->table_eff[ie-1][2];
	T1=paramrelic->table_eff[ie][0];
	T2=paramrelic->table_eff[ie-1][0];
			
	return (heff2-heff1)/(T2-T1)*(Temp-T1)+heff1;
}

/*--------------------------------------------------------------*/

double sgStar(double Temp, struct relicparam* paramrelic)
{
	int ie;
	
	if(Temp>= paramrelic->table_eff[0][0]) return paramrelic->table_eff[0][1];
	
	if(Temp<=0.) return paramrelic->table_eff[275][1];

	ie=1;
	while(Temp<paramrelic->table_eff[ie][0]) ie++;

	double sgStar1,sgStar2,T1,T2;
	sgStar1=paramrelic->table_eff[ie][1];
	sgStar2=paramrelic->table_eff[ie-1][1];
	T1=paramrelic->table_eff[ie][0];
	T2=paramrelic->table_eff[ie-1][0];
		
	return (sgStar2-sgStar1)/(T2-T1)*(Temp-T1)+sgStar1;
}

/*--------------------------------------------------------------*/

double geff(double Temp, struct relicparam* paramrelic)
{
	double heff0=heff(Temp,paramrelic);
	
	return pow(heff0/sgStar(Temp,paramrelic)*(1.+(heff(Temp*1.001,paramrelic)-heff(Temp*0.999,paramrelic))/0.006/heff0),2.);
}

/*--------------------------------------------------------------*/

void Init_cosmomodel(struct relicparam* paramrelic)
{
	paramrelic->eta0=6.19e-10; /* baryon-to-photon ratio from WMAP */
	paramrelic->life_neutron=885.7; /* neutron lifetime */
	paramrelic->nbnu=3.; /* number of neutrinos */

	paramrelic->dd0=paramrelic->ndd=paramrelic->Tdend=0.;
	paramrelic->sd0=paramrelic->nsd=paramrelic->Tsend=0.;
	paramrelic->nt0=paramrelic->nnt=paramrelic->Tnend=0.;
	paramrelic->Sigmad0=paramrelic->nSigmad=paramrelic->TSigmaend=0.;
	Init_modeleff(2,paramrelic);
	return;
}

/*--------------------------------------------------------------*/

void Init_cosmomodel_param(double eta, double nbnu, struct relicparam* paramrelic)
{
	paramrelic->eta0=eta;
	paramrelic->nbnu=nbnu;
	return;
}

/*--------------------------------------------------------------*/

void Init_dark_density(double dd0, double ndd, double T_end, struct relicparam* paramrelic)
{
	paramrelic->dd0=dd0;
	paramrelic->ndd=ndd;
	paramrelic->Tdend=T_end;
	
	return;
}

/*--------------------------------------------------------------*/

void Init_dark_entropy(double sd0, double nsd, double T_end, struct relicparam* paramrelic)
{
	paramrelic->sd0=sd0;
	paramrelic->nsd=nsd;
	paramrelic->Tsend=T_end;
	
	return;
}

/*--------------------------------------------------------------*/

void Init_dark_entropySigmaD(double Sigmad0, double nSigmad, double T_end, struct relicparam* paramrelic)
{
	paramrelic->Sigmad0=Sigmad0;
	paramrelic->nSigmad=nSigmad;
	paramrelic->TSigmaend=T_end;
	
	return;
}

/*--------------------------------------------------------------*/

void Init_nonthermal(double nt0, double nnt, double T_end, struct relicparam* paramrelic)
{
	paramrelic->nt0=nt0;
	paramrelic->nnt=nnt;
	paramrelic->Tnend=T_end;
	
	return;
}

/*--------------------------------------------------------------*/

double dark_density(double T, struct relicparam* paramrelic)
{
	if((paramrelic->dd0==0.)||(T<paramrelic->Tdend)) return 0.;
	
	double rho_rad_1MeV=pi*pi/30.*geff(1.e-3,paramrelic)*1.e-12;
	
	return paramrelic->dd0*rho_rad_1MeV*pow(T/1.e-3,paramrelic->ndd);
}

/*--------------------------------------------------------------*/

double dark_entropy(double T, struct relicparam* paramrelic)
{
	if((paramrelic->sd0==0.)&&(paramrelic->Sigmad0==0.)) return 0.;
	
	if((paramrelic->Sigmad0==0.)&&(T<paramrelic->Tsend)) return 0.;
	
	if(paramrelic->Sigmad0==0.)
	{
		double s_rad_1MeV=2.*pi*pi/45.*heff(1.e-3,paramrelic)*1.e-9;
	
		return paramrelic->sd0*s_rad_1MeV*pow(T/1.e-3,paramrelic->nsd);
	}
	else
	{
		double lnT,dlnT,Ttmp;
		int ie,nmax;
		double integ=0.;
		
		nmax=50;
		
		lnT=log(1.e-15);
		
		dlnT=(log(T)-lnT)/nmax;
		
		for(ie=1;ie<nmax;ie++) 
		{
			lnT+=dlnT;
			Ttmp=exp(lnT);
			integ+=sgStar(Ttmp,paramrelic)*dark_entropy_Sigmad(Ttmp,paramrelic)/sqrt(1.+dark_density(Ttmp,paramrelic)/(pi*pi/30.*geff(Ttmp,paramrelic)*pow(Ttmp,4.)))/pow(heff(Ttmp,paramrelic),2.)/pow(Ttmp,5.);
		}
		
integ+=sgStar(T,paramrelic)*dark_entropy_Sigmad(T,paramrelic)/sqrt(1.+dark_density(T,paramrelic)/(pi*pi/30.*geff(T,paramrelic)*pow(T,4.)))/pow(heff(T,paramrelic),2.)/pow(T,5.)/2.;
		
		integ*=dlnT;

		double Mplanck=1.2209e19;	

		return 3.*Mplanck*sqrt(5./4./pi/pi/pi)*heff(T,paramrelic)*T*T*T*integ;	
	}
}

/*--------------------------------------------------------------*/

double dark_entropy_derivative(double T, struct relicparam* paramrelic)
{
	if((paramrelic->sd0==0.)&&(paramrelic->Sigmad0==0.)) return 0.;
	
	if((paramrelic->Sigmad0==0.)&&(T<paramrelic->Tsend)) return 0.;
	
	if(paramrelic->Sigmad0==0.)
	{
		return (dark_entropy(T*1.001,paramrelic)-dark_entropy(T*0.999,paramrelic))/0.002/T;
	}
	else
	{
	
		double Mplanck=1.2209e19;	
		return 3.*sgStar(T,paramrelic)/T/heff(T,paramrelic)*(sqrt(geff(T,paramrelic))*dark_entropy(T,paramrelic)-sqrt(5.*Mplanck/4./pi/pi/pi)/T/T*dark_entropy_Sigmad(T,paramrelic)/sqrt(1.+dark_density(T,paramrelic)/(pi*pi/30.*geff(T,paramrelic)*pow(T,4.))));
	}
}

/*--------------------------------------------------------------*/

double dark_entropy_Sigmad(double T, struct relicparam* paramrelic)
{
	if((paramrelic->sd0==0.)&&(paramrelic->Sigmad0==0.)) return 0.;
	
	if((paramrelic->Sigmad0==0.)&&(T<paramrelic->Tsend)) return 0.;
	
	double Mplanck=1.2209e19;
	
	if(paramrelic->Sigmad0==0.)
	{	
		return 1./Mplanck*(sqrt(24.*pi*(pi*pi/30.*geff(T,paramrelic)*pow(T,4.)+dark_density(T,paramrelic)))*dark_entropy(T,paramrelic)-sqrt(4.*pi*pi*pi/45.)*heff(T,paramrelic)/sgStar(T,paramrelic)*pow(T,3)*sqrt(1.+dark_density(T,paramrelic)/(pi*pi/30.*geff(T,paramrelic)*pow(T,4.)))*dark_entropy_derivative(T,paramrelic));
	}
	else
	{
		if(T<paramrelic->TSigmaend) return 0.;

		double s_rad_1MeV=2.*pi*pi/45.*heff(1.e-3,paramrelic)*1.e-9;
	
		double Sigma_rad_1MeV= 1./Mplanck*sqrt(4.*pi*pi*pi/5.*geff(1.e-3,paramrelic))*(1.e-6)*s_rad_1MeV;
	
		return paramrelic->Sigmad0*Sigma_rad_1MeV*pow(T/1.e-3,paramrelic->nSigmad);
	}
}

/*--------------------------------------------------------------*/

double nonthermal(double T, struct relicparam* paramrelic)
{
	if((paramrelic->nt0==0.)||(T<paramrelic->Tnend)) return 0.;
	
	return paramrelic->nt0*1.e-50*pow(T/1.e-3,paramrelic->nnt);
}
